@@ -1,10 +1,14 @@
 # TODO: Save to ini, toml
+import json
+import shutil
 from collections import Counter
 from configparser import ConfigParser
+from datetime import timedelta
 from logging import getLogger
-from os import environ, symlink
-from os.path import relpath, splitext
+from os import environ
+from os.path import basename, relpath, splitext
 from pathlib import Path
+from string import Template
 from time import time
 
 import tomli
@@ -17,20 +21,25 @@ from ruamel.yaml.error import YAMLError
 
 from .. import __version__
 from ..constants import (
+    Status,
     AUTOMATION_NAME,
     AUTOMATION_ROUTE,
     AUTOMATION_VERSION,
     BATCH_ROUTE,
+    DEBUG_VARIABLE_DICTIONARIES,
     MODE_NAMES,
     PRINTER_BY_NAME,
     RUN_ROUTE,
     STYLE_ROUTE,
+    VARIABLE_ID_PATTERN,
+    VARIABLE_ID_TEMPLATE_PATTERN,
     VIEW_BY_NAME)
 from ..exceptions import (
     CrossComputeConfigurationError,
     CrossComputeConfigurationFormatError,
     CrossComputeConfigurationNotImplementedError,
     CrossComputeError)
+from ..macros.iterable import find_item
 from ..macros.package import is_equivalent_version
 from .printer import initialize_printer_by_name
 from .variable import (
@@ -68,6 +77,7 @@ class AutomationDefinition(Definition):
         self.path = path = Path(kwargs['path'])
         self.folder = path.parents[0]
         self.index = kwargs['index']
+        self.group_definitions = kwargs['group_definitions']
         self._validation_functions = [
             validate_protocol,
             validate_automation_identifiers,
@@ -82,12 +92,13 @@ class AutomationDefinition(Definition):
             validate_display_styles,
             validate_display_templates,
             validate_display_pages,
-            validate_print,
+            validate_authorization,
+            validate_prints,
         ]
 
     def get_variable_definitions(self, mode_name, with_all=False):
-        variable_definitions = self.variable_definitions_by_mode_name[
-            mode_name]
+        variable_definitions = self.variable_definitions_by_mode_name.get(
+            mode_name, [])
         if with_all:
             variable_definitions = variable_definitions.copy()
             for MODE_NAME in MODE_NAMES:
@@ -116,24 +127,15 @@ class AutomationDefinition(Definition):
 
     def get_design_name(self, page_id):
         design_name = DESIGN_NAMES_BY_PAGE_ID[page_id][0]
-        if page_id not in self.page_definition_by_id:
-            return design_name
-        page_definition = self.page_definition_by_id[page_id]
-        return page_definition.configuration.get('design', design_name)
-
-    def update_datasets(self):
-        automation_folder = self.folder
-        for dataset_definition in self.dataset_definitions:
-            if 'path' in dataset_definition.reference:
-                reference_path = dataset_definition.reference['path']
-                source_path = automation_folder / reference_path
-                target_path = automation_folder / dataset_definition.path
-                if target_path.is_symlink():
-                    target_path.unlink()
-                elif target_path.exists():
-                    continue
-                target_folder = target_path.parent
-                symlink(source_path.relative_to(target_folder), target_path)
+        if page_id in self.page_definition_by_id:
+            page_definition = self.page_definition_by_id[page_id]
+            design_name = page_definition.configuration.get(
+                'design', design_name)
+        elif page_id in MODE_NAMES:
+            variable_definitions = self.get_variable_definitions(page_id)
+            if not variable_definitions:
+                design_name = 'none'
+        return design_name
 
 
 class VariableDefinition(Definition):
@@ -166,6 +168,15 @@ class BatchDefinition(Definition):
             validate_batch_configuration,
         ]
 
+    def get_status(self):
+        status = Status.NEW
+        path = self.folder / 'debug' / 'variables.dictionary'
+        if path.exists():
+            d = json.load(path.open('rt'))
+            return_code = d['return_code']
+            status = Status.DONE if return_code == 0 else Status.FAILED
+        return status
+
 
 class DatasetDefinition(Definition):
 
@@ -189,27 +200,53 @@ class ScriptDefinition(Definition):
         command_string = self.command
         if command_string:
             return command_string
-        if 'path' not in self:
-            return
-        script_path = self.path
-        suffix = script_path.suffix
-        if suffix == '.ipynb':
-            folder = self.automation_folder / self.folder
-            old_path = folder / script_path
-            script_path = '.' + str(script_path.with_suffix('.ipynb.py'))
+        folder = self.automation_folder / self.folder
+        if 'path' in self:
+            script_path = self.path
+            suffix = script_path.suffix
+            if suffix == '.ipynb':
+                old_path = folder / script_path
+                script_path = '.' + str(script_path.with_suffix('.ipynb.py'))
+                new_path = folder / script_path
+                L.info(
+                    'exporting %s to %s in %s', self.path, script_path, folder)
+                try:
+                    script_text = PythonExporter().from_notebook_node(
+                        load_notebook(old_path, NO_CONVERT))[0]
+                    with new_path.open('wt') as script_file:
+                        script_file.write(script_text)
+                except Exception as e:
+                    raise CrossComputeConfigurationError(e)
+        elif 'function' in self:
+            script_path = '.run.py'
             new_path = folder / script_path
-            L.info('exporting %s to %s in %s', self.path, script_path, folder)
-            try:
-                script_text = PythonExporter().from_notebook_node(
-                    load_notebook(old_path, NO_CONVERT))[0]
-                with (new_path).open('wt') as script_file:
-                    script_file.write(script_text)
-            except Exception as e:
-                raise CrossComputeConfigurationError(e)
-        self.command = command_string = f'python "{script_path}"'
+            function_string = self['function']
+            with new_path.open('wt') as f:
+                f.write(RUN_PY.substitute({
+                    'module_name': function_string.split('.')[0],
+                    'function_string': function_string}))
+        else:
+            return
+        self.command = command_string = f'python3 "{script_path}"'
         return command_string
 
 
+class PackageDefinition(Definition):
+
+    def _initialize(self, kwargs):
+        self._validation_functions = [
+            validate_package_identifiers,
+        ]
+
+
+class PortDefinition(Definition):
+
+    def _initialize(self, kwargs):
+        self._validation_functions = [
+            validate_port_identifiers,
+        ]
+
+
 class StyleDefinition(Definition):
 
     def _initialize(self, kwargs):
@@ -227,6 +264,30 @@ class PageDefinition(Definition):
         ]
 
 
+class TokenDefinition(Definition):
+
+    def _initialize(self, kwargs):
+        self._validation_functions = [
+            validate_token_identifiers,
+        ]
+
+
+class GroupDefinition(Definition):
+
+    def _initialize(self, kwargs):
+        self._validation_functions = [
+            validate_group_identifiers,
+        ]
+
+
+class PermissionDefinition(Definition):
+
+    def _initialize(self, kwargs):
+        self._validation_functions = [
+            validate_permission_identifiers,
+        ]
+
+
 class PrintDefinition(Definition):
 
     def _initialize(self, kwargs):
@@ -256,14 +317,15 @@ def save_raw_configuration_yaml(configuration_path, configuration):
     return configuration_path
 
 
-def load_configuration(configuration_path, index=0):
+def load_configuration(configuration_path, index=0, group_definitions=[]):
     configuration_path = Path(configuration_path).absolute()
     configuration = load_raw_configuration(configuration_path)
     try:
         configuration = AutomationDefinition(
             configuration,
             path=configuration_path,
-            index=index)
+            index=index,
+            group_definitions=group_definitions)
     except CrossComputeConfigurationError as e:
         if not hasattr(e, 'path'):
             e.path = configuration_path
@@ -329,11 +391,13 @@ def validate_protocol(configuration):
 def validate_automation_identifiers(configuration):
     index = configuration.index
     name = configuration.get('name', make_automation_name(index))
+    description = configuration.get('description', '')
     version = configuration.get('version', AUTOMATION_VERSION)
     slug = configuration.get('slug', format_slug(name))
     uri = AUTOMATION_ROUTE.format(automation_slug=slug)
     return {
         'name': name,
+        'description': description,
         'version': version,
         'slug': slug,
         'uri': uri,
@@ -341,34 +405,34 @@ def validate_automation_identifiers(configuration):
 
 
 def validate_imports(configuration):
-    automation_configurations = []
-    remaining_configurations = [configuration]
-    while remaining_configurations:
-        c = remaining_configurations.pop(0)
-        folder = c.folder
-        import_configurations = get_dictionaries(c, 'imports')
-        for i, import_configuration in enumerate(import_configurations, 1):
-            if 'path' in import_configuration:
-                path = import_configuration['path']
-                try:
-                    automation_configuration = load_configuration(
-                        folder / path, index=i)
-                except CrossComputeConfigurationFormatError as e:
-                    raise CrossComputeConfigurationError(e)
-            else:
-                raise CrossComputeConfigurationError(
-                    'path required for each import')
-            remaining_configurations.append(automation_configuration)
-        automation_configurations.append(c)
+    automation_configurations = [configuration]
+    folder = configuration.folder
+    group_definitions = getattr(configuration, 'group_definitions', [])
+    import_configurations = get_dictionaries(configuration, 'imports')
+    for i, import_configuration in enumerate(import_configurations.copy(), 1):
+        if 'path' in import_configuration:
+            path = folder / import_configuration['path']
+            try:
+                automation_configuration = load_configuration(
+                    path, index=i, group_definitions=group_definitions)
+            except CrossComputeConfigurationFormatError as e:
+                raise CrossComputeConfigurationError(e)
+            import_configuration['path'] = path
+        else:
+            raise CrossComputeConfigurationError(
+                'path required for each import')
+        import_configurations.extend(
+            automation_configuration.import_configurations)
+        automation_configurations.extend(
+            automation_configuration.automation_definitions)
     automation_definitions = [
         _ for _ in automation_configurations if 'output' in _]
-    assert_unique_values(
-        [_.name for _ in automation_definitions],
-        'duplicate automation name {x}')
-    assert_unique_values(
-        [_.slug for _ in automation_definitions],
-        'duplicate automation slug {x}')
+    assert_unique_values([
+        _.name for _ in automation_definitions], 'duplicate name {x}')
+    assert_unique_values([
+        _.slug for _ in automation_definitions], 'duplicate slug {x}')
     return {
+        'import_configurations': import_configurations,
         'automation_definitions': automation_definitions,
     }
 
@@ -378,12 +442,16 @@ def validate_variables(configuration):
     view_names = set()
     for mode_name in MODE_NAMES:
         mode_configuration = get_dictionary(configuration, mode_name)
+        variable_dictionaries = get_dictionaries(
+            mode_configuration, 'variables')
+        if mode_name == 'debug':
+            variable_dictionaries[:0] = DEBUG_VARIABLE_DICTIONARIES
         variable_definitions = [VariableDefinition(
             _, mode_name=mode_name,
-        ) for _ in get_dictionaries(mode_configuration, 'variables')]
-        assert_unique_values(
-            [_.id for _ in variable_definitions],
-            f'duplicate variable id {{x}} in {mode_name}')
+        ) for _ in variable_dictionaries]
+        assert_unique_values([
+            _.id for _ in variable_definitions
+        ], f'duplicate variable id {{x}} in {mode_name}')
         variable_definitions_by_mode_name[mode_name] = variable_definitions
         view_names.update(_.view_name for _ in variable_definitions)
     L.debug('view_names = %s', list(view_names))
@@ -416,8 +484,8 @@ def validate_templates(configuration):
         template_definitions = [TemplateDefinition(
             _, automation_folder=automation_folder, mode_name=mode_name,
         ) for _ in get_dictionaries(mode_configuration, 'templates')]
-        assert_unique_values(
-            [_.id for _ in template_definitions],
+        assert_unique_values([
+            _.id for _ in template_definitions],
             f'duplicate template id {{x}} in {mode_name}')
         template_definitions_by_mode_name[mode_name] = template_definitions
     return {
@@ -433,15 +501,15 @@ def validate_batches(configuration):
     for raw_batch_definition in raw_batch_definitions:
         batch_definitions.extend(get_batch_definitions(
             raw_batch_definition, automation_folder, variable_definitions))
-    assert_unique_values(
-        [_.folder for _ in batch_definitions],
-        'duplicate batch folder {x}')
-    assert_unique_values(
-        [_.name for _ in batch_definitions],
-        'duplicate batch name {x}')
-    assert_unique_values(
-        [_.uri for _ in batch_definitions],
-        'duplicate batch uri {x}')
+    if 'output' in configuration and not(batch_definitions):
+        raise CrossComputeConfigurationError(
+            'no batches configured; please define at least one batch')
+    assert_unique_values([
+        _.folder for _ in batch_definitions], 'duplicate batch folder {x}')
+    assert_unique_values([
+        _.name for _ in batch_definitions], 'duplicate batch name {x}')
+    assert_unique_values([
+        _.uri for _ in batch_definitions], 'duplicate batch uri {x}')
     return {
         'batch_definitions': batch_definitions,
         'run_definitions': [],
@@ -449,21 +517,29 @@ def validate_batches(configuration):
 
 
 def validate_environment(configuration):
-    environment_configuration = get_dictionary(configuration, 'environment')
-    environment_variable_definitions = get_dictionaries(
-        environment_configuration, 'variables')
-    environment_variable_ids = get_environment_variable_ids(
-        environment_variable_definitions)
-    if environment_variable_ids:
-        L.debug('environment_variable_ids = %s', environment_variable_ids)
-    batch_concurrency_name = environment_configuration.get(
-        'batch', 'process').lower()
+    d = get_dictionary(configuration, 'environment')
+    engine_name = get_engine_name(d)
+    parent_image_name = d.get('image', 'python').strip()
+    package_definitions = [PackageDefinition(_) for _ in get_dictionaries(
+        d, 'packages')]
+    port_definitions = get_port_definitions(
+        d, configuration.get_variable_definitions('log', with_all=True))
+    environment_variable_ids = get_environment_variable_ids(get_dictionaries(
+        d, 'variables'))
+    batch_concurrency_name = d.get('batch', 'process').lower()
     if batch_concurrency_name not in ('process', 'thread', 'single'):
         raise CrossComputeConfigurationError(
             f'"{batch_concurrency_name}" batch concurrency is not supported')
+    interval_text = d.get('interval', '').strip()
+    interval_timedelta = get_interval_timedelta(interval_text)
     return {
+        'engine_name': engine_name,
+        'parent_image_name': parent_image_name,
+        'package_definitions': package_definitions,
+        'port_definitions': port_definitions,
         'environment_variable_ids': environment_variable_ids,
         'batch_concurrency_name': batch_concurrency_name,
+        'interval_timedelta': interval_timedelta,
     }
 
 
@@ -544,7 +620,25 @@ def validate_display_pages(configuration):
     }
 
 
-def validate_print(configuration):
+def validate_authorization(configuration):
+    authorization_dictionary = get_dictionary(configuration, 'authorization')
+    token_definitions = [TokenDefinition(_) for _ in get_dictionaries(
+        authorization_dictionary, 'tokens')]
+    identities_by_token = {
+        token: identities for _ in token_definitions
+        for token, identities in _.identities_by_token.items()}
+    group_definitions = [GroupDefinition(_) for _ in get_dictionaries(
+        authorization_dictionary, 'groups')]
+    if not group_definitions:
+        # Inherit group definitions from parent
+        group_definitions = getattr(configuration, 'group_definitions', [])
+    return {
+        'identities_by_token': identities_by_token,
+        'group_definitions': group_definitions,
+    }
+
+
+def validate_prints(configuration):
     print_definitions = [PrintDefinition(_) for _ in get_dictionaries(
         configuration, 'prints')]
     return {
@@ -569,13 +663,16 @@ def validate_template_identifiers(template_dictionary):
 
 
 def validate_variable_identifiers(variable_dictionary):
-    # TODO: Check that variable_id does not have quotes
     try:
         variable_id = variable_dictionary['id']
         view_name = variable_dictionary['view']
         variable_path = variable_dictionary['path']
     except KeyError as e:
         raise CrossComputeConfigurationError(f'{e} required for each variable')
+    if not VARIABLE_ID_PATTERN.match(variable_id):
+        raise CrossComputeConfigurationError(
+            f'{variable_id} is not a valid variable id; please use only '
+            'lowercase, uppercase, numbers, hyphens, underscores and spaces')
     if relpath(variable_path).startswith('..'):
         raise CrossComputeConfigurationError(
             f'path {variable_path} for variable {variable_id} must be within '
@@ -598,10 +695,10 @@ def validate_variable_configuration(variable_dictionary):
 def validate_batch_identifiers(batch_dictionary):
     is_run = batch_dictionary.is_run
     try:
-        folder = Path(get_scalar_text(batch_dictionary, 'folder'))
+        folder = get_scalar_text(batch_dictionary, 'folder')
     except KeyError as e:
         raise CrossComputeConfigurationError(f'{e} required for each batch')
-    name = get_scalar_text(batch_dictionary, 'name', folder.name)
+    name = get_scalar_text(batch_dictionary, 'name', basename(folder))
     slug = get_scalar_text(batch_dictionary, 'slug', name)
     data_by_id = batch_dictionary.data_by_id
     if data_by_id and not is_run:
@@ -616,7 +713,7 @@ def validate_batch_identifiers(batch_dictionary):
             if 'path' in batch_configuration:
                 e.path = batch_configuration['path']
             raise
-    d = {'folder': folder, 'name': name, 'slug': slug}
+    d = {'folder': Path(folder), 'name': name, 'slug': slug}
     if data_by_id:
         for k, v in d.items():
             if k in batch_dictionary:
@@ -641,20 +738,19 @@ def validate_batch_configuration(batch_dictionary):
 
 
 def validate_dataset_identifiers(dataset_dictionary):
-    path = Path(dataset_dictionary.get('path', '').strip())
     return {
-        'path': path,
+        'path': get_folder_plus_path(dataset_dictionary),
     }
 
 
 def validate_dataset_reference(dataset_dictionary):
     automation_folder = dataset_dictionary.automation_folder
     dataset_reference = get_dictionary(dataset_dictionary, 'reference')
-    if 'path' in dataset_reference:
-        source_path = Path(dataset_reference['path'].strip())
-        if not (automation_folder / source_path).exists():
+    reference_path = get_folder_plus_path(dataset_reference)
+    if reference_path:
+        if not (automation_folder / reference_path).exists():
             raise CrossComputeConfigurationError(
-                f'could not find dataset reference path {source_path}')
+                f'could not find dataset reference {reference_path}')
         target_path = dataset_dictionary.path
         if target_path.exists() and not target_path.is_symlink():
             raise CrossComputeConfigurationError(
@@ -689,6 +785,38 @@ def validate_script_identifiers(script_dictionary):
     }
 
 
+def validate_package_identifiers(package_dictionary):
+    try:
+        package_id = package_dictionary['id']
+        manager_name = package_dictionary['manager']
+    except KeyError as e:
+        raise CrossComputeConfigurationError(f'{e} required for each package')
+    if manager_name not in PACKAGE_MANAGER_NAMES:
+        raise CrossComputeConfigurationError(
+            f'"{manager_name}" manager is not supported')
+    return {
+        'id': package_id,
+        'manager_name': manager_name,
+    }
+
+
+def validate_port_identifiers(port_dictionary):
+    try:
+        port_id = port_dictionary['id']
+        port_number = port_dictionary['number']
+    except KeyError as e:
+        raise CrossComputeConfigurationError(f'{e} required for each port')
+    try:
+        port_number = int(port_number)
+    except ValueError:
+        raise CrossComputeConfigurationError(
+            f'{port_number} must be an integer')
+    return {
+        'id': port_id,
+        'number': port_number,
+    }
+
+
 def validate_style_identifiers(style_dictionary):
     uri = style_dictionary.get('uri', '').strip()
     path = style_dictionary.get('path', '').strip()
@@ -727,6 +855,71 @@ def validate_page_configuration(page_dictionary):
     }
 
 
+def validate_token_identifiers(token_dictionary):
+    try:
+        token_path = token_dictionary['path']
+    except KeyError as e:
+        raise CrossComputeConfigurationError(f'{e} required for each token')
+    token_path = Path(token_path)
+    suffix = token_path.suffix
+    if suffix == '.yml':
+        yaml = YAML()
+        d = yaml.load(token_path)
+    elif suffix == '.json':
+        d = json.load(token_path.open('rt'))
+    else:
+        raise CrossComputeConfigurationError(
+            f'{suffix} not supported for token paths')
+    identities_by_token = {}
+    for token, identities in d.items():
+        variable_match = VARIABLE_ID_TEMPLATE_PATTERN.match(token)
+        if variable_match:
+            variable_id = variable_match.group(1)
+            try:
+                token = environ[variable_id]
+            except KeyError:
+                e = CrossComputeConfigurationError(
+                    f'{variable_id} is missing in the environment')
+                e.path = token_path
+                raise e
+        identities_by_token[token] = identities
+    return {
+        'identities_by_token': identities_by_token,
+    }
+
+
+def validate_group_identifiers(group_dictionary):
+    try:
+        group_configuration = group_dictionary['configuration']
+    except KeyError as e:
+        raise CrossComputeConfigurationError(f'{e} required for each group')
+    group_permissions = [PermissionDefinition(_) for _ in get_dictionaries(
+        group_dictionary, 'permissions')]
+    return {
+        'configuration': group_configuration,
+        'permissions': group_permissions,
+    }
+
+
+def validate_permission_identifiers(permission_dictionary):
+    try:
+        permission_id = permission_dictionary['id']
+    except KeyError as e:
+        raise CrossComputeConfigurationError(
+            f'{e} required for each permission')
+    if permission_id not in PERMISSION_IDS:
+        raise CrossComputeConfigurationError(
+            f'"{permission_id}" permission not supported')
+    permission_action = permission_dictionary.get('action', 'accept')
+    if permission_action not in PERMISSION_ACTIONS:
+        raise CrossComputeConfigurationError(
+            f'"{permission_action}" action not supported')
+    return {
+        'id': permission_id,
+        'action': permission_action,
+    }
+
+
 def validate_print_identifiers(print_dictionary):
     initialize_printer_by_name()
     print_format = print_dictionary.get('format', '').strip()
@@ -814,6 +1007,39 @@ def get_template_text(
     return '\n'.join(template_texts)
 
 
+def get_engine_name(environment_dictionary):
+    engine_name = environment_dictionary.get('engine', 'unsafe').strip()
+    if engine_name == 'podman':
+        if not shutil.which('podman'):
+            L.warning('podman is not available on this machine')
+    elif engine_name == 'unsafe':
+        L.warning(
+            'using engine=unsafe; use engine=podman for untrusted code')
+    else:
+        raise CrossComputeConfigurationError(
+            f'"{engine_name}" engine is not supported')
+    return engine_name
+
+
+def get_port_definitions(environment_dictionary, variable_definitions):
+    port_definitions = [PortDefinition(_) for _ in get_dictionaries(
+        environment_dictionary, 'ports')]
+    for port_definition in port_definitions:
+        port_id = port_definition.id
+        try:
+            variable_definition = find_item(
+                variable_definitions, 'id', port_id)
+        except StopIteration:
+            raise CrossComputeConfigurationError(
+                f'{port_id} port must have a matching variable definition')
+        mode_name = variable_definition.mode_name
+        if mode_name not in ['log', 'debug']:
+            raise CrossComputeConfigurationError(
+                f'{port_id} port must correspond to a log or debug variable')
+        port_definition.mode_name = mode_name
+    return port_definitions
+
+
 def get_environment_variable_ids(environment_variable_definitions):
     variable_ids = set()
     for environment_variable_definition in environment_variable_definitions:
@@ -831,6 +1057,27 @@ def get_environment_variable_ids(environment_variable_definitions):
     return variable_ids
 
 
+def get_interval_timedelta(interval_text):
+    if not interval_text:
+        return
+    try:
+        count, name = interval_text.split()
+        count = int(count)
+    except ValueError:
+        raise CrossComputeConfigurationError(
+            f'unparseable interval "{interval_text}"; '
+            f'expected something like "30 minutes"')
+    for unit_name in INTERVAL_UNIT_NAMES:
+        if name.startswith(unit_name[:-1]):
+            break
+    else:
+        unit_names_text = ' '.join(INTERVAL_UNIT_NAMES)
+        raise CrossComputeConfigurationError(
+            f'unsupported interval unit "{name}" in "{interval_text}"; '
+            f'expected {unit_names_text}')
+    return timedelta(**{unit_name: count})
+
+
 def get_scalar_text(d, key, default=None):
     value = d.get(key) or default
     if value is None:
@@ -882,6 +1129,14 @@ def make_automation_name(automation_index):
     return AUTOMATION_NAME.replace('X', str(automation_index))
 
 
+def get_folder_plus_path(d):
+    folder = d.get('folder', '').strip()
+    path = d.get('path', '').strip()
+    if not folder and not path:
+        return
+    return Path(folder, path)
+
+
 def get_dictionaries(d, key):
     values = get_list(d, key)
     for value in values:
@@ -910,6 +1165,14 @@ def assert_unique_values(xs, message):
             raise CrossComputeConfigurationError(message.format(x=x))
 
 
+PACKAGE_MANAGER_NAMES = [
+    'apt',
+    'dnf',
+    'npm',
+    'pip',
+]
+
+
 DESIGN_NAMES_BY_PAGE_ID = {
     'automation': ['input', 'output', 'none'],
     'input': ['flex-vertical', 'none'],
@@ -917,4 +1180,39 @@ DESIGN_NAMES_BY_PAGE_ID = {
     'log': ['flex-vertical', 'none'],
     'debug': ['flex-vertical', 'none'],
 }
+
+
+INTERVAL_UNIT_NAMES = 'seconds', 'minutes', 'hours', 'days', 'weeks'
+
+
+RUN_PY = Template('''\
+import inspect
+from os import getenv
+from pathlib import Path
+
+import $module_name
+
+
+folder_by_name = {}
+for x in 'input_folder', 'output_folder', 'log_folder', 'debug_folder':
+    folder_by_name[x] = Path(getenv('CROSSCOMPUTE_' + x.upper()))
+d = {}
+for x in inspect.getargspec(run.plot).args:
+    d[x] = folder_by_name[x]
+$function_string(**d)''')
+
+
+PERMISSION_IDS = [
+    'add_authorization',
+    'see_root',
+    'see_automation',
+    'see_batch',
+    'run_automation',
+]
+PERMISSION_ACTIONS = [
+    'accept',
+    'match',
+]
+
+
 L = getLogger(__name__)
