@@ -3,23 +3,25 @@ import json
 import shutil
 from dataclasses import dataclass
 from logging import getLogger
-from os.path import basename, exists
 from string import Template
+from urllib.request import urlretrieve as download_uri
 
 from importlib_metadata import entry_points
 from invisibleroads_macros_log import format_path
 from invisibleroads_macros_text import format_name, format_slug
 
 from ..constants import (
+    CACHED_FILE_SIZE_LIMIT_IN_BYTES,
     MAXIMUM_FILE_CACHE_LENGTH,
     TEMPLATES_FOLDER,
-    VARIABLE_ID_PATTERN,
+    VARIABLE_ID_TEMPLATE_PATTERN,
     VIEW_BY_NAME)
 from ..exceptions import (
     CrossComputeConfigurationError,
     CrossComputeConfigurationNotImplementedError,
     CrossComputeDataError)
 from ..macros.disk import FileCache
+from ..macros.iterable import find_item
 from ..macros.package import import_attribute
 from ..macros.web import (
     escape_quotes_html,
@@ -31,7 +33,7 @@ from .interface import Batch
 class Element():
 
     id: str
-    base_uri: str
+    root_uri: str
     mode_name: str
     design_name: str
     for_print: bool
@@ -62,21 +64,24 @@ class VariableView():
     def parse(self, data):
         return data
 
+    def process(self, path):
+        pass
+
     def render(self, b: Batch, x: Element):
         if x.mode_name == 'input':
             render = self.render_input
         else:
             render = self.render_output
-        page_dictionary = render(b, x)
-        main_text = page_dictionary['main_text']
+        jinja_dictionary = render(b, x)
+        main_text = jinja_dictionary['main_text']
         if x.design_name != 'none':
             if main_text.endswith('</a>') or main_text.endswith('</span>'):
                 tag_name = 'span'
             else:
                 tag_name = 'div'
-            page_dictionary['main_text'] = '<%s class="_view">%s</%s>' % (
+            jinja_dictionary['main_text'] = '<%s class="_view">%s</%s>' % (
                 tag_name, main_text, tag_name)
-        return page_dictionary
+        return jinja_dictionary
 
     def render_input(self, b: Batch, x: Element):
         return {
@@ -105,7 +110,7 @@ class LinkView(VariableView):
         element_id = x.id
         data_uri = b.get_data_uri(variable_definition, x)
         c = b.get_variable_configuration(variable_definition)
-        file_name = c.get('file-name', basename(self.variable_path))
+        file_name = c.get('file-name', self.variable_path.name)
         link_text = c.get('link-text', file_name)
         main_text = (
             f'<a id="{element_id}" href="{data_uri}" '
@@ -144,7 +149,7 @@ class StringView(VariableView):
         if 'value' in data:
             value = data['value']
         elif 'path' in data:
-            value = FILE_TEXT_CACHE[data['path']]
+            value = load_file_text(data['path'])
         else:
             value = ''
         return value
@@ -237,7 +242,7 @@ class EmailView(StringView):
     input_type = 'email'
 
 
-class TextView(StringView):
+class TextView(VariableView):
 
     view_name = 'text'
 
@@ -400,6 +405,45 @@ class TableView(VariableView):
         }
 
 
+class FrameView(VariableView):
+
+    view_name = 'frame'
+
+    def render_output(self, b: Batch, x: Element):
+        variable_definition = self.variable_definition
+        variable_id = self.variable_id
+        element_id = x.id
+        data_uri = b.get_data_uri(variable_definition, x)
+        c = b.get_variable_configuration(variable_definition)
+        data = b.get_data(variable_definition)
+        if 'value' in data:
+            value = data['value']
+        else:
+            value = ''
+        main_text = (
+            f'<iframe id="{element_id}" '
+            f'class="_{self.mode_name} _{self.view_name} {variable_id}" '
+            f'src="{escape_quotes_html(value)}" frameborder="0">'
+            '</iframe>')
+        if x.design_name not in ['none']:
+            main_text = add_label_html(main_text, c, variable_id, element_id)
+
+        js_texts = [
+            FRAME_JS_HEADER,
+            FRAME_JS_OUTPUT.substitute({
+                'variable_id': variable_id,
+                'element_id': element_id,
+                'data_uri': data_uri,
+            }),
+        ]
+        return {
+            'css_uris': [],
+            'js_uris': [],
+            'main_text': main_text,
+            'js_texts': js_texts,
+        }
+
+
 def initialize_view_by_name():
     for entry_point in entry_points().select(group='crosscompute.views'):
         VIEW_BY_NAME[entry_point.name] = import_attribute(entry_point.value)
@@ -407,6 +451,7 @@ def initialize_view_by_name():
 
 
 def save_variable_data(target_path, data_by_id, variable_definitions):
+    target_path.parent.mkdir(parents=True, exist_ok=True)
     variable_data_by_id = get_variable_data_by_id(
         variable_definitions, data_by_id)
     if target_path.suffix == '.dictionary':
@@ -418,12 +463,17 @@ def save_variable_data(target_path, data_by_id, variable_definitions):
         raise CrossComputeConfigurationError(
             'use file extension .dictionary for multiple variables')
     else:
-        variable_data = list(variable_data_by_id.values())[0]
+        variable_id, variable_data = list(variable_data_by_id.items())[0]
         if 'value' in variable_data:
             open(target_path, 'wt').write(variable_data['value'])
         elif 'path' in variable_data:
             shutil.copy(variable_data['path'], target_path)
-        # TODO: Download variable_data['uri']
+        elif 'uri' in variable_data:
+            download_uri(variable_data['uri'], target_path)
+        variable_definition = find_item(
+            variable_definitions, 'id', variable_id)
+        variable_view = VariableView.get_from(variable_definition)
+        variable_view.process(target_path)
 
 
 def get_data_by_id_from_folder(folder, variable_definitions):
@@ -495,22 +545,35 @@ def parse_data_by_id(data_by_id, variable_definitions):
 
 
 def update_variable_data(target_path, data_by_id):
+    target_path.parent.mkdir(parents=True, exist_ok=True)
     try:
-        if exists(target_path):
-            with open(target_path, 'r+t') as f:
-                d = json.load(f)
-                d.update(data_by_id)
-                f.seek(0)
-                f.truncate()
-                json.dump(d, f)
+        if target_path.suffix == '.dictionary':
+            if target_path.exists():
+                with target_path.open('r+t') as f:
+                    d = json.load(f)
+                    d.update(data_by_id)
+                    f.seek(0)
+                    f.truncate()
+                    json.dump(d, f)
+            else:
+                with target_path.open('wt') as f:
+                    d = data_by_id
+                    json.dump(d, f)
         else:
-            with open(target_path, 'wt') as f:
-                d = data_by_id
-                json.dump(d, f)
+            with target_path.open('wt') as f:
+                f.write(data_by_id.values()[0])
     except (json.JSONDecodeError, OSError) as e:
         raise CrossComputeDataError(e)
 
 
+def process_variable_data(path, variable_definition):
+    variable_id = variable_definition.id
+    variable_view = VariableView.get_from(variable_definition)
+    variable_data = load_variable_data(path, variable_id)
+    variable_view.process(path)
+    return variable_data
+
+
 def load_variable_data(path, variable_id):
     file_data = FILE_DATA_CACHE[path]
     if path.suffix == '.dictionary':
@@ -527,18 +590,39 @@ def load_variable_data(path, variable_id):
 
 
 def load_file_data(path):
-    if path.suffix == '.dictionary':
-        try:
-            value = json.load(path.open('rt'))
-        except (json.JSONDecodeError, OSError) as e:
-            raise CrossComputeDataError(
-                f'could not load {format_path(path)}: {e}')
-        return {'value': value}
     if not path.exists():
         raise CrossComputeDataError(f'could not find {format_path(path)}')
+    suffix = path.suffix
+    if suffix == '.dictionary':
+        return load_dictionary_data(path)
+    if suffix == '.txt':
+        return load_text_data(path)
     return {'path': path}
 
 
+def load_dictionary_data(path):
+    try:
+        value = json.load(path.open('rt'))
+    except (json.JSONDecodeError, OSError) as e:
+        raise CrossComputeDataError(
+            f'could not load {format_path(path)}: {e}')
+    if not isinstance(value, dict):
+        raise CrossComputeDataError(
+            f'expected dictionary in {format_path(path)}')
+    return {'value': value}
+
+
+def load_text_data(path):
+    try:
+        if path.stat().st_size > CACHED_FILE_SIZE_LIMIT_IN_BYTES:
+            return {'path': path}
+        value = load_file_text(path)
+    except OSError as e:
+        raise CrossComputeDataError(
+            f'could not load {format_path(path)}: {e}')
+    return {'value': value}
+
+
 def load_file_text(path):
     return path.read_text().rstrip()
 
@@ -592,7 +676,7 @@ def format_text(text, data_by_id):
                 f'{e} function not supported in "{text}"')
         return str(value)
 
-    return VARIABLE_ID_PATTERN.sub(f, text)
+    return VARIABLE_ID_TEMPLATE_PATTERN.sub(f, text)
 
 
 def apply_functions(value, function_names, function_by_name):
@@ -659,6 +743,10 @@ TABLE_JS_HEADER = load_view_text('tableHeader.js')
 TABLE_JS_OUTPUT = Template(load_view_text('tableOutput.js'))
 
 
+FRAME_JS_HEADER = load_view_text('frameHeader.js')
+FRAME_JS_OUTPUT = Template(load_view_text('frameOutput.js'))
+
+
 YIELD_DATA_BY_ID_BY_EXTENSION = {
     '.csv': yield_data_by_id_from_csv,
     '.txt': yield_data_by_id_from_txt,
@@ -668,6 +756,3 @@ YIELD_DATA_BY_ID_BY_EXTENSION = {
 FILE_DATA_CACHE = FileCache(
     load_file_data=load_file_data,
     maximum_length=MAXIMUM_FILE_CACHE_LENGTH)
-FILE_TEXT_CACHE = FileCache(
-    load_file_data=load_file_text,
-    maximum_length=MAXIMUM_FILE_CACHE_LENGTH)
