@@ -1,9 +1,11 @@
 from argparse import ArgumentParser
 from logging import getLogger
+from os import getenv
 
 from crosscompute.exceptions import (
     CrossComputeError)
-from crosscompute.routines.automation import DiskAutomation
+from crosscompute.routines.automation import (
+    DiskAutomation)
 from crosscompute.routines.log import (
     configure_argument_parser_for_logging,
     configure_logging_from)
@@ -19,6 +21,7 @@ def do(arguments=None):
     args = a.parse_args(arguments)
     try:
         configure_logging_from(args)
+        configure_running_from(args)
         automation = DiskAutomation.load(args.path_or_folder)
     except CrossComputeError as e:
         L.error(e)
@@ -27,21 +30,30 @@ def do(arguments=None):
 
 
 def configure_argument_parser_for_running(a):
-    # TODO: Implement clean
-    '''
     a.add_argument(
-        '--clean', dest='with_clean', action='store_true',
-        help='delete batch folders before running')
-    '''
+        '--no-rebuild', dest='with_rebuild', action='store_false',
+        help='do not rebuild container images')
+
+
+def configure_running_from(args):
+    port = getattr(args, 'port', None)
+    origin_uri = getenv('CROSSCOMPUTE_ORIGIN_URI') or (
+        f'http://localhost:{port}' if port else 'http://localhost')
+    args.environment = {
+        'CROSSCOMPUTE_ORIGIN_URI': origin_uri,
+    }
 
 
 def run_with(automation, args):
-    return run(automation)
+    return run(automation, args.environment, args.with_rebuild)
 
 
-def run(automation):
+def run(
+        automation,
+        environment,
+        with_rebuild=True):
     try:
-        automation.run()
+        automation.run(environment, with_rebuild)
     except CrossComputeError as e:
         L.error(e)
     except KeyboardInterrupt:
