@@ -9,13 +9,18 @@ from crosscompute.constants import (
     PORT)
 from crosscompute.exceptions import (
     CrossComputeError)
-from crosscompute.macros.web import is_port_in_use, open_browser
-from crosscompute.routines.automation import DiskAutomation
+from crosscompute.macros.web import (
+    find_open_port, is_port_in_use, open_browser)
+from crosscompute.routines.automation import (
+    DiskAutomation)
 from crosscompute.routines.log import (
     configure_argument_parser_for_logging,
     configure_logging_from)
 from crosscompute.scripts.configure import (
     configure_argument_parser_for_configuring)
+from crosscompute.scripts.run import (
+    configure_argument_parser_for_running,
+    configure_running_from)
 
 
 def do(arguments=None):
@@ -23,10 +28,12 @@ def do(arguments=None):
     configure_argument_parser_for_logging(a)
     configure_argument_parser_for_configuring(a)
     configure_argument_parser_for_serving(a)
+    configure_argument_parser_for_running(a)
     args = a.parse_args(arguments)
     try:
         configure_logging_from(args)
         configure_serving_from(args)
+        configure_running_from(args)
         automation = DiskAutomation.load(args.path_or_folder)
     except CrossComputeError as e:
         L.error(e)
@@ -42,21 +49,21 @@ def configure_argument_parser_for_serving(a):
         help='specify * to listen for requests from all ip addresses')
     a.add_argument(
         '--port', metavar='X',
-        default=PORT,
+        default=find_open_port(PORT),
         help='specify port to listen to for requests')
     a.add_argument(
         '--no-browser', dest='with_browser', action='store_false',
         help='do not open browser')
     a.add_argument(
-        '--static', dest='is_static', action='store_true',
-        help='disable page update on file change')
+        '--no-refresh', dest='with_refresh', action='store_false',
+        help='do not refresh page on file change')
     a.add_argument(
-        '--production', dest='is_production', action='store_true',
-        help='disable server restart on file change')
+        '--no-restart', dest='with_restart', action='store_false',
+        help='do not restart server on file change')
     a.add_argument(
-        '--base-uri', metavar='X',
+        '--root-uri', metavar='X',
         default='',
-        help='specify base uri for all routes')
+        help='specify root uri for all routes')
     a.add_argument(
         '--origins', metavar='X', nargs='+', dest='allowed_origins',
         default=[],
@@ -68,45 +75,53 @@ def configure_argument_parser_for_serving(a):
     a.add_argument(
         '--disk-debounce', metavar='X', type=int,
         default=DISK_DEBOUNCE_IN_MILLISECONDS,
-        help='interval in milliseconds to wait until the disk stops changing')
+        help='interval in milliseconds to wait until changes stop')
 
 
 def configure_serving_from(args):
-    base_uri = args.base_uri
-    if base_uri and not base_uri.startswith('/'):
-        args.base_uri = '/' + base_uri
+    root_uri = args.root_uri
+    if root_uri and not root_uri.startswith('/'):
+        args.root_uri = '/' + root_uri
 
 
 def serve_with(automation, args):
     return serve(
         automation,
-        args.host,
-        args.port,
-        args.with_browser,
-        args.is_static,
-        args.is_production,
-        args.base_uri,
-        args.allowed_origins,
-        args.disk_poll,
-        args.disk_debounce)
+        environment=args.environment,
+        host=args.host,
+        port=args.port,
+        with_browser=args.with_browser,
+        with_refresh=args.with_refresh,
+        with_restart=args.with_restart,
+        root_uri=args.root_uri,
+        allowed_origins=args.allowed_origins,
+        disk_poll_in_milliseconds=args.disk_poll,
+        disk_debounce_in_milliseconds=args.disk_debounce)
 
 
 def serve(
-        automation, host=HOST, port=PORT, with_browser=True,
-        is_static=False, is_production=False,
-        base_uri='', allowed_origins=None,
+        automation,
+        environment,
+        host=HOST,
+        port=PORT,
+        with_browser=False,
+        with_refresh=False,
+        with_restart=False,
+        root_uri='',
+        allowed_origins=None,
         disk_poll_in_milliseconds=DISK_POLL_IN_MILLISECONDS,
         disk_debounce_in_milliseconds=DISK_DEBOUNCE_IN_MILLISECONDS):
     try:
         if with_browser and 'DISPLAY' in environ:
             L.info('opening browser; set --no-browser to disable')
-            open_browser(f'http://localhost:{port}{base_uri}')
+            open_browser(f'http://localhost:{port}{root_uri}')
         automation.serve(
+            environment,
             host=host,
             port=port,
-            is_static=is_static,
-            is_production=is_production,
-            base_uri=base_uri,
+            with_refresh=with_refresh,
+            with_restart=with_restart,
+            root_uri=root_uri,
             allowed_origins=allowed_origins,
             disk_poll_in_milliseconds=disk_poll_in_milliseconds,
             disk_debounce_in_milliseconds=disk_debounce_in_milliseconds)
